
import sys
import os
import tensorflow as tf
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from sann.bin import config
from sann.bin import data_utils





class Model(tf.keras.Model):
    def __init__(self,trainable,initial_weigh):
        super().__init__()
        self.large = tf.keras.layers.Dense(300)
        self.large2 = tf.keras.layers.Dense(800)
        self.dropout = tf.keras.layers.Dropout(config.config.drop_rate)
        self.out_layer = tf.keras.layers.Dense(data_utils.utils.cla_num,activation='softmax')

    def call(self, x):
        x = self.large(x)
        x = self.large2(x)
        x = self.dropout(x)
        x = self.out_layer(x)
        return x


import pandas as pd
import os


class DescribingData:
    def __init__(self, old_dir):
        self.data = self.data = pd.read_excel(old_dir, header=0)  # 第一行默认为表头，数据第一行必须有表头
        self.save_dir = os.path.dirname(old_dir)

    def to_medical_his(self):
        data = self.data.copy()
        save_path = os.path.join(self.save_dir, 'medical_history.xlsx')

        for column in list(data.columns.values):
            if {0,1} == set(data.loc[:,column].tolist()):
                data.loc[data[column]==1,[column]] = column

        data.to_excel(save_path)




import docx
import os
import sys
sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

class CollatingWords():
    """
    This class, named 'collating words' , is built to collate some words from JH's doc through
    removing duplication, merging similar unit and adjusting the sequence of units.

    The basic operating unit of this class is unit, composed by some words which are similar in
    meaning or spelling.
    """

    def __init__(self,old_dir):
        self.old_dir = old_dir
        self.old_doc = docx.Document(old_dir)
        self.units = None
        self.get_units()  #从docx里读取unit

    def get_units(self):
        """
        从当前类的docx对象中读取units的函数，没有返回值，直接赋值到类属性。
        :return:
        """

        # units ,with a nature of list,is used to store unit
        units = []

        # units ,with a nature of dict,is used to store words and its meanings.
        unit = {}

        # unit是按段落分开的，一个段落就是一个unit
        for paragraph in self.old_doc.paragraphs:

            # unit与unit之间有一个空段落，因此只要读到‘’ or ‘ ’ 且之前读到过具体内容，就是一个unit结束了
            if paragraph.text in ['', ' ']:
                if unit:   #检验是否之前读到过具体内容，如果是，则将这个unit添加进units列表里面
                    unit['index'] = list(unit.keys())      #为了方便后续操作，将unit的key也就是单词做成一个index。
                    units.append(unit.copy())
                unit = {}

            else:   #如果读到的不是空段落，则证明读到的是一个unit 里的其中一行。  一行就是一个单词+ ‘ ’ +解释

                if 'Unit:' in paragraph.text : continue  # 跳过标题行

                word_meaning = paragraph.text.strip().split(' ', 1)  # 用空格分隔开单词和解释
                word = word_meaning[0].lower()       # 将单词统一转化为小写

                if word == '':continue              # 如果单词是‘ ’ 则证明这是一个无效的单词，跳过当前循环

                try:                                # 如果存在单词，但不存在解释，则自动补充解释为 待补充
                    meaning = word_meaning[1].strip()
                except IndexError:
                    meaning = '待补充'

                unit[word] = meaning               # 讲这个单词极其解释以key-value的形式加入到当前unit中


        self.units = units

    def save_cdoc(self):
        """
        这个函数是用来保存整理之后的units到doc的，没有返回值，保存路径与读取路径一致。
        """

        new_doc = docx.Document()

        for i, unit in enumerate(self.units):

            new_doc.add_paragraph('Unit:' + str(i))
            for key, value in unit.items():
                if key == 'index': continue
                new_doc.add_paragraph(key + ' '*5 + value)
            new_doc.add_paragraph()


        new_dir = os.path.dirname(self.old_dir)
        new_dir = os.path.join(new_dir,'new.docx')

        new_doc.save(new_dir)

    def rp(self):
        """
        this function, completely named removing duplication, is built to removing some lists
        which are identical to one of forward lists
        """

        dup_unit_index = []       # 要删除的重复unit的索引列表
        unit_words = []           # 将unit的index的值（一个包含该unit所有单词的列表）储存到一个列表

        for index, unit in enumerate(self.units):     # 循环遍历所有unit
            if unit['index'] not in unit_words:       # 判断当前unit的index是否已经存在，如果存在就是重复的unit
                unit_words.append(unit['index'])      # 如果没有不存在，则没有重复，将这个index加入到列表中。
            else:
                dup_unit_index.append(index)         # 将重复unit的索引添加进要删除的重复unit的索引列表

        dup_unit_index.sort(reverse=True)            # 逆序排列，这样从后往前按索引删除列表元素，删除一个之后，其余的要删除元素的索引不会改变。
        for i in dup_unit_index:
            print('删除重复unit：{}'.format(self.units[i]['index']))       # 打印要删除的元素
            del self.units[i]                        # 删除元素

    def rs(self):

        """
        这个函数是 removing single word。如果一个unit只有一个单词，那没什么意义，直接删除。
        """
        single_unit_index = []

        for index, unit in enumerate(self.units):
            if len(unit['index']) <=1 :                   # 如果index 的长度<=1,证明该unit最多只含有一个单词
                single_unit_index.append(index)

        single_unit_index.sort(reverse=True)              # 删除方法与去重一样。
        for i in single_unit_index:
            print('删除仅有一个单词的unit：{}'.format(self.units[i]['index']))
            del self.units[i]

    def sort_by_corr(self):
        """
        将相关的unit组成一个unit集合，按不同集合的unit数量进行逆序排序，然后赋值到类units中。（方便记忆的时候区块化记忆）
        :return:
        """

        def get_similar_uid(ori_uid):
            """
            这个函数在units 中找出与参数unit相关的unit
            参数：目标unit的索引id
            返回值：相关unit的索引id列表，每一个元素就是一个相关unit的id

            """

            similar_uid = []

            for uid, unit in enumerate(self.units):      # 遍历unit

                if uid == ori_uid: continue              # 如果当前遍历的unit就是目标unit，则跳过

                # 这个求相似的逻辑，现在是按照单词完全一样，后期可以改成一个函数，只要有60%子字符串一致就算相似。
                if len(set(self.units[uid]['index']).intersection(set(self.units[ori_uid]['index']))) >= 1:
                    similar_uid.append(uid)     # 如果相似，则将当前遍历的unit的索引id加入到相似列表

            return similar_uid

        similar_units_list = []        # 相似unit组成的units列表为元素，组成的一个列表。列表元素是相似units列表，相似units列表的元素是unit
        similar_uid_list = []          # 当前彼此相似unit的id索引列表

        for cur_id, cur_unit in enumerate(self.units):  # 遍历units

            next = 0                                    # 与后续的next代码一起实现，如果当前unit已经存在于某个相似units之中，则跳过当前循环
            for similar_units in similar_units_list:    # 从相似units列表中遍历相似units
                for unit in similar_units:              # 从相似units中遍历unit
                    if cur_unit == unit: next = 1       # 如果当前unit 已经存在于某个相似units之中，则跳过当前循环

            if next == 1: continue


            similar_uid_list.append(cur_id)                    # 如果当前unit不存在于任何一个相似units，则添加到一个新相似units中
            similar_uid_list.extend(get_similar_uid(cur_id))   # 将与这unit相关的unit添加到这个相似units中

            i = len(similar_uid_list) - 1                      # 设置一个阈值i，如果经过一次相关寻找相关unit之后，这个相似units中
                                                               # unit的数量并没有增加，则证明所有与之相关的unit都找出来了，则结束循环。

            while len(similar_uid_list) > i :                  # 如果相似units 大于阈值，也就是改变了，则进入循环
                i = len(similar_uid_list)                      # 首先赋值阈值等于units的长度，这样如果units的长度没有改变，则会结束循环

                temp_similar_uid = similar_uid_list.copy()     # 复制一个当前units，这样可以在遍历循环的同时，进行改变
                for uid in temp_similar_uid :                  # 遍历当前相似units 中每一个unit
                    similar_uid_list.extend(get_similar_uid(uid)) # 找出每一个unit的相似unit并加入到这个当前units中
                    similar_uid_list = list(set(similar_uid_list)) # 这样会有重复，因此取集合以排除重复。

            temp_similar_units = [self.units[i] for i in similar_uid_list]  # 通过uid，把这个相似units中的所有unit添加到一个暂时units列表中
            similar_units_list.append(temp_similar_units)    # 把这个units的暂时列表当成一个元素添加到相似units列表中
            similar_uid_list = []

        similar_units_list = sorted(similar_units_list,key=lambda x: len(x), reverse=True)     # 逆序排序

        temp_similar_units = []      # 赋值给类的units
        for units in similar_units_list:
            temp_similar_units.extend(units)
        self.units = temp_similar_units

    def run(self):
        self.rp()
        self.rs()
        self.sort_by_corr()
        self.save_cdoc()




import importlib

import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['SimHei']
from os import path, stat
import tensorflow as tf
from sklearn.model_selection import cross_val_score
import pickle
import sklearn
import numpy as np
import os
import sys
sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
config = importlib.import_module('config')
data_utils = importlib.import_module('data_utils')
model_utils = importlib.import_module('model_utils')


class Utils():
    def __init__(self):
        pass

    def comparison_model(self,model_name_list ):
        with open(config.config.score_save_path, 'rb') as f:
            scores = pickle.load(f)

        for model_name in model_name_list:
            print(model_name)
            for ill_name in config.config.task_name_list:
                print(ill_name)
                for metric_name in config.config.metrics_list:
                    print('{:.2f}'.format(sum(scores[model_name][ill_name][metric_name])/len(scores[model_name][ill_name][metric_name])))

    def get_arguments(self,model_name):

        with open('./data/{}/{}_ini.pkl'.format(config.config.task_name_list[0], model_name), 'rb') as f:
            arguments = pickle.load(f)
            print(','.join(list(arguments.keys())))

        for ill_name in config.config.task_name_list:
            print(ill_name)
            with open('./data/{}/{}_ini.pkl'.format(ill_name,model_name),'rb') as f:
                arguments = pickle.load(f)

                for argument in list(arguments.values()):
                    print(argument,end=',')

                print()

    def plot_epochs_scores(self,epochs_num,task_path,force_refresh=False):


        save_test_loss_path = self.__get_pic_save_path(task_path,path.basename(task_path)+'-'+'test_loss'+'.jpg')


        save_test_accuracy_path = self.__get_pic_save_path(task_path,path.basename(task_path) + '-' + 'test_accuracy' + '.jpg')


        if path.exists(save_test_loss_path) and path.exists(save_test_accuracy_path):
            print('epochs相关图像已存在，跳过......')
            return None

        plt_y_loss = {'SANN':np.array([0 for i in range(epochs_num)],dtype='float64'),'ANN':np.array([0 for i in range(epochs_num)],dtype='float64')}
        plt_y_accuracy = {'SANN':np.array([0 for i in range(epochs_num)],dtype='float64'),'ANN':np.array([0 for i in range(epochs_num)],dtype='float64')}

        for cur_data in data_utils.utils.get_xy_cross_val(task_path,config.config.cross_val_num):
            train_x, train_y, test_x, test_y, cur_weigh =cur_data


            for model_name in ['SANN','ANN']:


                model = model_utils.utils.get_model(model_name,task_path,initial_weigh=cur_weigh)
                model.compile(loss='categorical_crossentropy', optimizer='adam',
                              metrics=[tf.keras.metrics.CategoricalAccuracy()
                                       ])

                history = model.fit(
                    x=train_x,
                    y=train_y,
                    validation_split=0.2,
                    batch_size=config.config.batch_size,
                    epochs=epochs_num)

                plt_y_loss[model_name] += np.array(history.history['val_loss'])
                plt_y_accuracy[model_name] += np.array(history.history['val_categorical_accuracy'])



        plot_x = [i for i in range(1, epochs_num + 1)]
        plt.plot(plot_x, plt_y_loss['SANN']/config.config.cross_val_num, label='SANN')
        plt.plot(plot_x, plt_y_loss['ANN']/config.config.cross_val_num, label='ANN')
        plt.legend(loc=config.config.legend_loc)
        plt.title(path.basename(task_path)+'-'+'test_loss')

        if force_refresh:
            plt.savefig(save_test_loss_path)

        plt.show()


        plot_x = [i for i in range(1, epochs_num + 1)]
        plt.plot(plot_x, plt_y_accuracy['SANN']/config.config.cross_val_num, label='SANN')
        plt.plot(plot_x, plt_y_accuracy['ANN']/config.config.cross_val_num, label='ANN')
        plt.ylim(0,1)
        plt.legend(loc=config.config.legend_loc)
        plt.title(path.basename(task_path)+'-'+'test_accuracy')

        if force_refresh:
            plt.savefig(save_test_accuracy_path)

        plt.show()

    def plot_model2ill_score(self,metrics,task_path,force_refresh=False):
        with open(self.__get_scores_path(task_path), 'rb') as f:
            score = pickle.load(f)

        plt_x = [i for i in range(1,config.config.cross_val_num+1)]


        for model_name in config.config.model_list:

            plt.plot(plt_x, score[model_name][metrics],
                         label=model_name+':'+str(round(sum(score[model_name][metrics])/len(score[model_name][metrics]),2)))


        plt.ylim(0.6, 1)
        plt.title(os.path.basename(task_path)+':Macro_{}'.format(metrics) )
        plt.legend()

        if force_refresh:
            plt.savefig( self.__get_pic_save_path(task_path,os.path.basename(task_path) + '_Macro_{}.png'.format(metrics)))
        plt.show()

    def __get_model2ill_scores(self,model_name,task_path,k_num,force_refresh):

        score_dict = {}
        acc_score_list = []
        pre_score_list = []
        rec_score_list = []
        f1_score_list = []


        for i, cur_data in enumerate(data_utils.utils.get_xy_cross_val(task_path=task_path,k=k_num)):


            train_x, train_y, test_x, test_y, cur_weigh = cur_data


            if model_name in ['ANN', 'SANN']:

                temp_save_path = path.join(path.join(path.join(task_path, model_name) , '{}'.format(i)),'')


                model = model_utils.utils.get_model(model_name,task_path,initial_weigh=cur_weigh)

                model.compile(loss='categorical_crossentropy', optimizer='adam',
                              metrics=[tf.keras.metrics.CategoricalAccuracy()
                                       ])

                if path.exists(temp_save_path) and not force_refresh:
                    callbacks = []
                    config.config.epochs_num = 1

                else:
                    callbacks = [
                    tf.keras.callbacks.ModelCheckpoint(filepath=temp_save_path, save_best_only=True,save_weights_only=True,monitor='val_categorical_accuracy',mode='max'),
                    tf.keras.callbacks.EarlyStopping(patience=config.config.patience_num, min_delta=1e-3)
                    ]

                model.fit(
                    x=train_x,
                    y=train_y,
                    validation_data=(test_x, test_y),
                    batch_size=config.config.batch_size,
                    epochs=config.config.epochs_num,
                    callbacks=callbacks)


                model.load_weights(temp_save_path)


                pre_y = model.predict(test_x)
                pre_y = data_utils.utils.onhot2id(pre_y)
                test_y = data_utils.utils.onhot2id(test_y)

            else:

                temp_save_path = path.join(task_path, model_name + '{}_for_score.pkl'.format(i))

                train_y = data_utils.utils.onhot2id(train_y)

                test_y = data_utils.utils.onhot2id(test_y)

                model = model_utils.utils.get_model(model_name, task_path)

                if path.exists(temp_save_path) and not force_refresh:
                    with open(temp_save_path, 'rb') as f:
                        model = pickle.load(f)
                else:
                    model.fit(train_x, train_y)
                    with open(temp_save_path,'wb') as f:
                        pickle.dump(model,f)

                pre_y = model.predict(test_x)


            acc_score_list.append(sklearn.metrics.accuracy_score(test_y,pre_y))

            pre_score_list.append(sklearn.metrics.precision_score(test_y,pre_y, average='macro'))

            rec_score_list.append(sklearn.metrics.recall_score(test_y,pre_y, average='macro'))

            f1_score_list.append(sklearn.metrics.f1_score(test_y,pre_y, average='macro'))



        score_dict['accuracy'] = acc_score_list
        score_dict['precision'] = pre_score_list
        score_dict['recall'] = rec_score_list
        score_dict['f1'] = f1_score_list

        return score_dict

    def reset_model2ill_scores(self,model_name,task_name,k_num):

        force_refresh = True

        task_path = path.join('./data',task_name)

        score_dict = {}
        acc_score_list = []
        pre_score_list = []
        rec_score_list = []
        f1_score_list = []

        for i, cur_data in enumerate(data_utils.utils.get_xy_cross_val(task_path=task_path, k=k_num)):


            train_x, train_y, test_x, test_y, cur_weigh = cur_data

            if model_name in ['ANN', 'SANN']:

                temp_save_path = path.join(path.join(path.join(task_path, model_name) , '{}'.format(i)),'')

                model = model_utils.utils.get_model(model_name, task_path, initial_weigh=cur_weigh)

                model.compile(loss='categorical_crossentropy', optimizer='adam',
                              metrics=[tf.keras.metrics.CategoricalAccuracy()
                                       ])

                if path.exists(temp_save_path) and not force_refresh:
                    callbacks = [

                    ]
                else:
                    callbacks = [
                        tf.keras.callbacks.ModelCheckpoint(filepath=temp_save_path, save_best_only=True,
                                                           save_weights_only=True,monitor='val_categorical_accuracy',mode='max'),
                        tf.keras.callbacks.EarlyStopping(patience=config.config.patience_num, min_delta=1e-3)
                    ]

                model.fit(
                    x=train_x,
                    y=train_y,
                    validation_data=(test_x, test_y),
                    batch_size=config.config.batch_size,
                    epochs=config.config.epochs_num,
                    callbacks=callbacks)

                model.load_weights(temp_save_path)

                pre_y = model.predict(test_x)
                pre_y = data_utils.utils.onhot2id(pre_y)
                test_y = data_utils.utils.onhot2id(test_y)

            else:

                temp_save_path = path.join(task_path, model_name + '{}_for_score.pkl'.format(i))

                train_y = data_utils.utils.onhot2id(train_y)

                test_y = data_utils.utils.onhot2id(test_y)

                model = model_utils.utils.get_model(model_name, task_path)

                if path.exists(temp_save_path) and not force_refresh:
                    with open(temp_save_path, 'rb') as f:
                        model = pickle.load(f)
                else:
                    model.fit(train_x, train_y)
                    with open(temp_save_path, 'wb') as f:
                        pickle.dump(model, f)

                pre_y = model.predict(test_x)

            acc_score_list.append(sklearn.metrics.accuracy_score(test_y, pre_y))

            pre_score_list.append(sklearn.metrics.precision_score(test_y, pre_y, average='macro'))

            rec_score_list.append(sklearn.metrics.recall_score(test_y, pre_y, average='macro'))

            f1_score_list.append(sklearn.metrics.f1_score(test_y, pre_y, average='macro'))

        score_dict['accuracy'] = acc_score_list
        score_dict['precision'] = pre_score_list
        score_dict['recall'] = rec_score_list
        score_dict['f1'] = f1_score_list

        with open(config.config.score_save_path, 'rb') as f:
            score = pickle.load(f)

            score[model_name][path.basename(task_path)] = score_dict

        with open(config.config.score_save_path, 'wb') as f:
            pickle.dump(score, f)

        return score_dict

    def __get_scores_path(self,task_path):
        return os.path.join(os.path.join(config.config.input_path,os.path.basename(task_path)),'scores.pkl')

    def __get_pic_save_path(self,task_path,file_name):
        if not os.path.exists(os.path.join(os.path.join(config.config.input_path, os.path.basename(task_path)),'doc')):
            os.mkdir(os.path.join(os.path.join(config.config.input_path, os.path.basename(task_path)),'doc'))
        return os.path.join(os.path.join(os.path.join(config.config.input_path, os.path.basename(task_path)), 'doc'),
                     file_name)

    def get_models2ill_scores(self,task_path, k_num,re_get_score=False,force_refresh=False):

        save_path = self.__get_scores_path(task_path)


        if path.exists(save_path) and not re_get_score:
            if stat(save_path).st_size > 0:
                print('scores已存在，跳过......')

                with open(save_path, 'rb') as f:
                    models_scores_dict = pickle.load(f)

                return models_scores_dict

        models_scores_dict = {}      # {model_name:{metrics:[k_score]}}

        for model_name in config.config.model_list:
            model_scores = self.__get_model2ill_scores(model_name,task_path, k_num, force_refresh)

            models_scores_dict[model_name] = model_scores

        with open(save_path,'wb') as f:
            pickle.dump(models_scores_dict,f)

        return models_scores_dict










utils = Utils()

if __name__ == '__main__':
    pass




import pandas as pd
import os
import numpy as np

class SelectingFeatures():

    def __init__(self, old_dir, standardization=None, balance=False):
        self.data = self.data = pd.read_excel(old_dir,header=0)   # 第一行默认为表头，数据第一行必须有表头
        self.save_dir = os.path.dirname(old_dir)       # 默认保存路径为原数据同一目录
        self.x = self.data.iloc[:,:-1]
        self.y = self.data.iloc[:, -1]

        if balance:
            self.__balancing_sample()

        if standardization == 'MM':
            from sklearn.preprocessing import MinMaxScaler
            self.x = MinMaxScaler().fit_transform(self.x)
        elif standardization == 'SS':
            from sklearn.preprocessing import StandardScaler
            self.x = StandardScaler().fit_transform(self.x)

    def __balancing_sample(self):
        from imblearn.over_sampling import SMOTE
        sm = SMOTE(random_state=100)
        self.x, self.y = sm.fit_sample(self.x, self.y)

    def save_weight(self,name,weight,index=None):
        save_path = os.path.join(self.save_dir, 'weight')
        if not os.path.exists(save_path):
            os.makedirs(save_path)

        save_path = os.path.join(save_path, '{}.xlsx'.format(name))

        if index == None:
            index = self.data.iloc[:,:-1].columns.values


        weight = np.array(weight).reshape(-1,1)
        weight_df =  pd.DataFrame(data=weight, index=index, columns=['weight'])
        weight_df.sort_values('weight', inplace=True,ascending=False)

        weight_df.to_excel(save_path)
        return weight_df

    def filter_variance(self):
        from sklearn.feature_selection import VarianceThreshold
        vt = VarianceThreshold().fit(self.x,self.y)
        variances = vt.variances_
        return self.save_weight('variances', variances)

    def filter_correlation(self,how='pearsonr'):

        if how == 'pearsonr':
            correlation = self.data.corr(method='spearman').iloc[:-1,-1]

        return self.save_weight('cor_{}'.format(how), correlation)

    def filter_mutual_info(self,how='mRMR'):

        def get_entropy(variables):
            from collections import Counter

            var_times_dict = Counter(variables)
            var_list = set(variables)
            all_times = len(variables)
            entropy = 0

            for x in var_list:
                p = var_times_dict[x] / all_times
                log = np.log2(p)
                entropy -= p * log

            return entropy

        def get_conditional_entropy(variables,cond_vars):
            from collections import Counter

            cond_var_times_dict = Counter(cond_vars)
            cond_var_set = set(cond_vars)
            entropy = 0

            for cond_var in cond_var_set:

                p1 = cond_var_times_dict[cond_var] / len(cond_vars)

                var_by_cvar = []
                for i,cond_var2 in enumerate(cond_vars):
                    if cond_var == cond_var2:
                        var_by_cvar.append(variables[i])

                var_times_dict = Counter(var_by_cvar)
                var_set = set(var_by_cvar)

                temp = 0
                for var in var_set:
                    p2 = var_times_dict[var] / len(var_by_cvar)
                    log2 = np.log2(p2)
                    temp += p2 * log2

                entropy -= p1 * temp

            return entropy

        def get_mul_info(variables,cond_vars):
            return get_entropy(variables) - get_conditional_entropy(variables,cond_vars)

        def mRMR(M,S,c):

            sequences = []

            if S.shape[1] == 0:
                m_name_list = list(M.columns.values)
                for m_name in m_name_list:
                    m = M.loc[:, m_name]
                    score = get_mul_info(m, c)
                    sequences.append((m_name, score))

                return sequences



            m_name_list = list(M.columns.values)
            s_name_list = list(S.columns.values)

            for m_name in m_name_list:

                m = M.loc[:,m_name]


                s_sum = 0
                for s_name in s_name_list:
                    s = S.loc[:, s_name]
                    s_sum += get_mul_info(m,s)

                s_sum *= 1/len(S)

                score = get_mul_info(m,c) - s_sum

                sequences.append((m_name,score))

            return sequences

        def get_features(how=how):

            if how == 'mRMR':
                get_score = mRMR

            c = self.y
            M = pd.DataFrame(data=self.x,columns=self.data.iloc[:,:-1].columns.values)
            S = pd.DataFrame()

            all_x = self.x.shape[1]

            outcome = []

            while S.shape[1] < all_x:
                M_score = get_score(M, S, c)
                sorted(M_score,key=lambda x:x[1],reverse=True)

                selected_m = M_score[0][0]
                S = pd.concat([S,M.loc[:,selected_m]],axis=1)
                del M[selected_m]

                outcome.append(M_score[0])

            weight = [i[1] for i in outcome]
            index = [i[0] for i in outcome]
            self.save_weight('mulinfo_mRMR', weight, index)

        get_features(how)

    def wrapper_sfs(self,estimator_name='SVC',direction='forward'):
        import numbers
        import numpy as np
        from sklearn.feature_selection._base import SelectorMixin
        from sklearn.base import BaseEstimator, MetaEstimatorMixin, clone
        from sklearn.utils._tags import _safe_tags
        from sklearn.utils.validation import check_is_fitted
        from sklearn.model_selection import cross_val_score

        class SequentialFeatureSelector(SelectorMixin, MetaEstimatorMixin,
                                        BaseEstimator):
            """Transformer that performs Sequential Feature Selection.

            This Sequential Feature Selector adds (forward selection) or
            removes (backward selection) features to form a feature subset in a
            greedy fashion. At each stage, this estimator chooses the best feature to
            add or remove based on the cross-validation score of an estimator.

            Read more in the :ref:`User Guide <sequential_feature_selection>`.

            .. versionadded:: 0.24

            Parameters
            ----------
            estimator : estimator instance
                An unfitted estimator.

            n_features_to_select : int or float, default=None
                The number of features to select. If `None`, half of the features are
                selected. If integer, the parameter is the absolute number of features
                to select. If float between 0 and 1, it is the fraction of features to
                select.

            direction: {'forward', 'backward'}, default='forward'
                Whether to perform forward selection or backward selection.

            scoring : str, callable, list/tuple or dict, default=None
                A single str (see :ref:`scoring_parameter`) or a callable
                (see :ref:`scoring`) to evaluate the predictions on the test set.

                NOTE that when using custom scorers, each scorer should return a single
                value. Metric functions returning a list/array of values can be wrapped
                into multiple scorers that return one value each.

                If None, the estimator's score method is used.

            cv : int, cross-validation generator or an iterable, default=None
                Determines the cross-validation splitting strategy.
                Possible inputs for cv are:

                - None, to use the default 5-fold cross validation,
                - integer, to specify the number of folds in a `(Stratified)KFold`,
                - :term:`CV splitter`,
                - An iterable yielding (train, test) splits as arrays of indices.

                For integer/None inputs, if the estimator is a classifier and ``y`` is
                either binary or multiclass, :class:`StratifiedKFold` is used. In all
                other cases, :class:`KFold` is used.

                Refer :ref:`User Guide <cross_validation>` for the various
                cross-validation strategies that can be used here.

            n_jobs : int, default=None
                Number of jobs to run in parallel. When evaluating a new feature to
                add or remove, the cross-validation procedure is parallel over the
                folds.
                ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
                ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
                for more details.

            Attributes
            ----------
            n_features_to_select_ : int
                The number of features that were selected.

            support_ : ndarray of shape (n_features,), dtype=bool
                The mask of selected features.

            See Also
            --------
            RFE : Recursive feature elimination based on importance weights.
            RFECV : Recursive feature elimination based on importance weights, with
                automatic selection of the number of features.
            SelectFromModel : Feature selection based on thresholds of importance
                weights.

            Examples
            --------
            >>> from sklearn.feature_selection import SequentialFeatureSelector
            >>> from sklearn.neighbors import KNeighborsClassifier
            >>> from sklearn.datasets import load_iris
            >>> X, y = load_iris(return_X_y=True)
            >>> knn = KNeighborsClassifier(n_neighbors=3)
            >>> sfs = SequentialFeatureSelector(knn, n_features_to_select=3)
            >>> sfs.fit(X, y)
            SequentialFeatureSelector(estimator=KNeighborsClassifier(n_neighbors=3),
                                      n_features_to_select=3)
            >>> sfs.get_support()
            array([ True, False,  True,  True])
            >>> sfs.transform(X).shape
            (150, 3)
            """

            def __init__(self, estimator, *, n_features_to_select=None,
                         direction='forward', scoring=None, cv=5, n_jobs=None):

                self.estimator = estimator
                self.n_features_to_select = n_features_to_select
                self.direction = direction
                self.scoring = scoring
                self.cv = cv
                self.n_jobs = n_jobs

            def fit(self, X, y):
                """Learn the features to select.

                Parameters
                ----------
                X : array-like of shape (n_samples, n_features)
                    Training vectors.
                y : array-like of shape (n_samples,)
                    Target values.

                Returns
                -------
                self : object
                """
                tags = self._get_tags()
                X, y = self._validate_data(
                    X, y, accept_sparse="csc",
                    ensure_min_features=2,
                    force_all_finite=not tags.get("allow_nan", True),
                    multi_output=True
                )
                n_features = X.shape[1]

                error_msg = ("n_features_to_select must be either None, an "
                             "integer in [1, n_features - 1] "
                             "representing the absolute "
                             "number of features, or a float in (0, 1] "
                             "representing a percentage of features to "
                             f"select. Got {self.n_features_to_select}")
                if self.n_features_to_select is None:
                    self.n_features_to_select_ = n_features // 2
                elif isinstance(self.n_features_to_select, numbers.Integral):
                    if not 0 < self.n_features_to_select < n_features:
                        raise ValueError(error_msg)
                    self.n_features_to_select_ = self.n_features_to_select
                elif isinstance(self.n_features_to_select, numbers.Real):
                    if not 0 < self.n_features_to_select <= 1:
                        raise ValueError(error_msg)
                    self.n_features_to_select_ = int(n_features *
                                                     self.n_features_to_select)
                else:
                    raise ValueError(error_msg)

                if self.direction not in ('forward', 'backward'):
                    raise ValueError(
                        "direction must be either 'forward' or 'backward'. "
                        f"Got {self.direction}."
                    )

                cloned_estimator = clone(self.estimator)

                # the current mask corresponds to the set of features:
                # - that we have already *selected* if we do forward selection
                # - that we have already *excluded* if we do backward selection
                current_mask = np.zeros(shape=n_features, dtype=bool)

                self.ranking = list(np.zeros(shape=n_features, dtype=int))

                n_iterations = (
                    self.n_features_to_select_ if self.direction == 'forward'
                    else n_features - self.n_features_to_select_
                )
                for _ in range(n_iterations):
                    new_feature_idx = self._get_best_new_feature(cloned_estimator, X,
                                                                 y, current_mask)
                    current_mask[new_feature_idx] = True

                    if self.direction != 'backward':
                        self.ranking[new_feature_idx] = n_features - _
                    else:
                        self.ranking[new_feature_idx] = -(n_features - _)

                if self.direction == 'backward':
                    current_mask = ~current_mask
                self.support_ = current_mask

                return self

            def _get_best_new_feature(self, estimator, X, y, current_mask):
                # Return the best new feature to add to the current_mask, i.e. return
                # the best new feature to add (resp. remove) when doing forward
                # selection (resp. backward selection)
                candidate_feature_indices = np.flatnonzero(~current_mask)
                scores = {}
                for feature_idx in candidate_feature_indices:
                    candidate_mask = current_mask.copy()
                    candidate_mask[feature_idx] = True
                    if self.direction == 'backward':
                        candidate_mask = ~candidate_mask
                    X_new = X[:, candidate_mask]
                    scores[feature_idx] = cross_val_score(
                        estimator, X_new, y, cv=self.cv, scoring=self.scoring,
                        n_jobs=self.n_jobs).mean()
                return max(scores, key=lambda feature_idx: scores[feature_idx])

            def _get_support_mask(self):
                check_is_fitted(self)
                return self.support_

            def _more_tags(self):
                return {
                    'allow_nan': _safe_tags(self.estimator, key="allow_nan"),
                    'requires_y': True,
                }

        if estimator_name == 'SVC':
            from sklearn.svm import SVC
            estimator = SVC()

        if estimator_name == 'KNN':
            from sklearn.neighbors import KNeighborsClassifier
            estimator = KNeighborsClassifier()

        sfs = SequentialFeatureSelector(estimator, direction=direction, n_features_to_select=2,).fit(self.x, self.y)


        weight = sfs.ranking

        return self.save_weight('wrapper_{}'.format(estimator_name),weight)

    def embedded_rfe(self,estimator_name='LSVC'):
        from sklearn.feature_selection import RFE

        if estimator_name == 'LSVC':
            from sklearn.svm import LinearSVC
            estimator = LinearSVC()
        elif estimator_name == 'LASSO':
            from sklearn.linear_model import LassoCV
            estimator = LassoCV()



        ref = RFE(estimator=estimator, n_features_to_select=1, step=1)
        ref.fit(self.x, self.y)
        weight = ref.ranking_

        n = self.x.shape[1]

        reverse_weight = []
        order_dict = dict(zip(range(1,n+1,1),range(n,0,-1)))
        for i in weight:
            reverse_weight.append(order_dict[i])

        self.save_weight('embedded_rfe_{}'.format(estimator_name), reverse_weight)

    def embedded_lasso(self):
        from sklearn.linear_model import LassoCV

        # x = StandardScaler().fit_transform(x)  # lasso必须先标准化

        lasso = LassoCV(cv=5) # 最优alpha竟然是0.00012463621602719477，所有千万不要自己设置字典，就用自动的才行
        lasso.fit(self.x, self.y)
        weight = lasso.coef_
        self.save_weight('wrapper_lasso', weight)

    def embedded_gbdt(self):
        from sklearn.ensemble import GradientBoostingClassifier
        from sklearn.model_selection import GridSearchCV


        param = {'learning_rate': [0.1,0.2,0.3],
                 'n_estimators': [i for i in range(100,500,100)],
                 'min_samples_split':[i for i in range(2,4,1)],
                 'max_depth':[i for i in range(3,4,1)]}
        grid_search = GridSearchCV(GradientBoostingClassifier(),param , scoring='accuracy', cv=None)
        grid_search.fit(self.x, self.y)


        model = grid_search.best_estimator_.fit(self.x, self.y)
        weight = model.feature_importances_
        self.save_weight('embedded_gbdt', weight)

    def embedded_rf(self):
        from sklearn.ensemble import RandomForestClassifier
        from sklearn.model_selection import GridSearchCV

        param = {'n_estimators': [i for i in range(100,500,100)],
                 'min_samples_split':[i for i in range(2,4,1)],
                 'max_depth':[i for i in range(3,4,1)]}

        grid_search = GridSearchCV(RandomForestClassifier(),param , scoring='accuracy', cv=None)
        grid_search.fit(self.x, self.y)


        model = grid_search.best_estimator_.fit(self.x, self.y)
        weight = model.feature_importances_
        self.save_weight('embedded_rf', weight)

    def embedded_sann(self):
        pass




import importlib
import numpy as np
import pandas as pd
import os
import sys
import tensorflow as tf
from sklearn.utils import shuffle
from importlib import import_module
import pickle
sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
config = importlib.import_module('config')

import configparser

class Utils:
    def __init__(self):
        cg = configparser.ConfigParser()
        cg.read(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'initial.ini'),
                encoding='utf-8')
        self.cur_task_path = cg.get('task','cur_task_path')

        if self.cur_task_path != 'None':
            self.__get_data(self.cur_task_path)


    def __get_data(self,task_path):
        cg = configparser.ConfigParser()
        cg.read(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))),'initial.ini'), encoding='utf-8')
        cg.set('task', 'cur_task_path', task_path)


        with open(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))),'initial.ini'), 'w', encoding='utf-8') as f:
            cg.write(f)

        file_name_list = os.listdir(task_path)
        data_path = None

        for file_name in file_name_list:
            if 'dataset' in file_name:
                data_path = os.path.join(task_path, file_name)

        if not data_path:
            print('没有发现病案文件，请确保为*.xlsx格式并在当前目录下！')



        data = pd.read_excel(data_path,header=0)
        data.dropna(axis=0,how='any',inplace=True)
        data = data.reset_index(drop=True)
        data = shuffle(data,random_state=config.config.random_num)


        x_name = data.columns.values

        self.x_name = list(x_name)[:-1]
        self.x_num = len(self.x_name)

        self.data = np.array(data)

        self.cla_name = list(set(self.data[:,-1]))
        self.cla_num = len(self.cla_name)

        self.cur_data = self.data  # 在生成权重时候，会以此数据为基准。 仅在交叉验证时，这个值会被动态赋值成训练集。其他时候都为全数据集

    def __get_fre_x2c(self, x_id, cla_name):
        data = self.cur_data[self.cur_data[:, -1] == cla_name][:, :-1]
        frq_x2c = data[:, x_id].sum()
        return frq_x2c

        # 获取在所有样本中，指定x的频次

    def __get_frq_x2C(self, x_id):
        data = self.cur_data[:, :-1]
        frq_x2C = data[:, x_id].sum()
        return frq_x2C

        # 获取在指定分类下，所有X的平均频次

    def __get_frq_X2c(self, cla_name):
        data = pd.DataFrame(self.cur_data[self.cur_data[:, -1] == cla_name][:, :-1])
        frq_X2c = 0
        for i in range(data.shape[1]):
            frq_X2c += np.array(data)[:, i].sum()
        return frq_X2c

        # 获取在所有样本里，所有X的平均频次

    def __get_frq_X2C(self):
        data = pd.DataFrame(self.cur_data[:, :-1])
        frq_X2C = 0
        for i in range(data.shape[1]):
            frq_X2C += np.array(data)[:, i].sum()
        return frq_X2C

    def __get_x2c(self, x_id, cla_name):
        x2c = self.__get_fre_x2c(x_id, cla_name)
        w_c = self.cur_data[self.cur_data[:, -1] == cla_name][:, :-1].shape[0]
        h_c = w_c / 2

        return (x2c - h_c) / w_c

    def __get_x2C(self, x_id):
        x2C = self.__get_frq_x2C(x_id)
        w_C = self.cur_data[:, :-1].shape[0]
        h_C = w_C / 2
        return (x2C - h_C) / w_C

    def __get_X2c(self, cla_name):
        X2c = self.__get_frq_X2c(cla_name)
        w_c = self.cur_data[self.cur_data[:, -1] == cla_name][:, :-1].shape[0] * \
              self.cur_data[self.cur_data[:, -1] == cla_name][:, :-1].shape[1]
        h_c = w_c / 2
        return (X2c - h_c) / w_c

    def __get_X2C(self):
        X2C = self.__get_frq_X2C()
        w_C = self.cur_data[:, :-1].shape[0] * self.cur_data[:, :-1].shape[1]
        h_C = w_C / 2
        return (X2C - h_C) / w_C

    def __get_x2all(self,x_id,):
        x2all_list = []
        x2all_list_debug = []
        obtained_X2C = self.__get_X2C()
        obtained_x2C = self.__get_x2C(x_id)
        for i in range(self.cla_num):

            cla_name = self.cla_name[i]
            obtained_X2c = self.__get_X2c(cla_name)

            x2all_list.append(   (
                            (self.__get_x2c(x_id,cla_name) - obtained_x2C) * (1+abs(obtained_X2C-0.5))
                              +
                              (obtained_X2c - obtained_X2C)

                                )
                            )


            x2all_list_debug.append('{},{},{},{}'.format(
                self.__get_x2c(x_id,cla_name),
                obtained_x2C,
                obtained_X2c,
                obtained_X2C
                )

            )

        return x2all_list, x2all_list_debug

    def __get_all2all(self):


        weigh = pd.DataFrame(np.zeros(shape=(self.x_num, self.cla_num)),columns=self.cla_name,index=self.x_name)
        weigh_debug = pd.DataFrame(np.zeros(shape=(self.x_num, self.cla_num)),columns=self.cla_name,index=self.x_name)

        for x_id in range(self.x_num):
            x2all_list, x2all_list_debug = self.__get_x2all(x_id)

            weigh.iloc[x_id, :] = np.array(x2all_list)
            weigh_debug.iloc[x_id, :] = np.array(x2all_list_debug)

        return weigh, weigh_debug

    def get_weigh_corr(self,task_path,force_refresh=False):


        x,y = self.get_xy_onehot(task_path)

        save_path = os.path.join(os.path.join(task_path,'weigh') ,'{}_weigh.ckpt'.format('SANN'))
        weigh_path = os.path.join(task_path, '{}_weigh.xlsx'.format('SANN'))
        corr_path = os.path.join(task_path, '{}_corr.xlsx'.format('SANN'))

        if os.path.exists(weigh_path) and not force_refresh:
            if os.stat(weigh_path).st_size > 0:
                print('{}-{}权值及相关系数已存在，跳过......'.format('SANN',os.path.basename(task_path)))
                return None


        cur_weigh, debug_weigh = self.__get_all2all()
        corr = self.get_corr(cur_weigh)

        model_file = import_module('model.{}'.format('SANN'))
        model = model_file.Model(trainable=True, initial_weigh=[corr, cur_weigh])


        model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=[tf.keras.metrics.CategoricalAccuracy()
                                                                                  ])
        callbacks = [
            tf.keras.callbacks.ModelCheckpoint(filepath=save_path, save_best_only=True,save_weights_only=True),
            tf.keras.callbacks.EarlyStopping(patience=config.config.patience_num, min_delta=1e-3)
        ]

        history = model.fit(
            x=x,
            y=y,
            validation_split=0.2,
            batch_size=config.config.batch_size,
            epochs=config.config.epochs_num,
            callbacks=callbacks)

        model.load_weights(save_path)


        pd.DataFrame(model.get_weights()[1],
                 columns=self.cla_name,
                 index=self.x_name).to_excel(weigh_path)

        pd.DataFrame(model.get_weights()[0],
                 columns=self.x_name,
                 index=self.x_name).to_excel(corr_path)

        print('{}-{}权值及相关系数生成完毕'.format('SANN', os.path.basename(task_path)))

    def get_corr(self,cur_weigh,force_refresh=False):

        # # 皮尔逊相关系数
        # data = pd.DataFrame(self.cur_data[:, :-1],
        #                     columns=self.x_name)
        # corr = data.corr()
        #
        # corr.fillna(0, inplace = True)

        # 利用对分类的贡献度的相似性生成相关系数
        data = pd.DataFrame(cur_weigh,
                            index=self.x_name,
                            columns=self.cla_name)
        corr = pd.DataFrame(np.zeros(shape=[self.x_num,self.x_num]),
                            index=self.x_name,
                            columns=self.x_name)

        for cla_name in self.cla_name:
            cur_data = np.array(data.loc[:,cla_name])
            for x_id in range(self.x_num):

                x_x_cla_temp = []

                x_value = cur_data[x_id]
                max_difference_v = max(abs(x_value-np.max(cur_data)),abs(x_value-np.min(cur_data)))

                for x_id_again in range(self.x_num):
                    difference_value = abs(cur_data[x_id] - cur_data[x_id_again])

                    # 判断是否同号
                    if max(cur_data[x_id],0) == max(cur_data[x_id_again],0):

                        corr_value = 1- (difference_value/max_difference_v)

                    else:
                        corr_value = - (difference_value/max_difference_v)

                    x_x_cla_temp.append(corr_value)

                corr.iloc[x_id,:] += x_x_cla_temp

        corr /= self.cla_num

        return np.array(corr)

    def get_xy_cross_val(self,task_path,k,force_refresh=False):

        save_path = os.path.join(task_path,'cross_val_data.pkl')



        self.__get_data(task_path)

        xy_cross_val_list = []

        if os.path.exists(save_path) and not force_refresh:
            with open(save_path,'rb') as f:
                xy_cross_val_list = pickle.load(f)
            return xy_cross_val_list


        def get_xy_split(test_index):

            x = self.data[:, :-1]
            y = tf.keras.utils.to_categorical(self.data[:, -1]-1)

            train_x = np.delete(x,test_index,axis=0)
            test_x = x[test_index,:]

            train_y = np.delete(y,test_index,axis=0)
            test_y = y[test_index,:]

            self.cur_data = np.delete(self.data,test_index,axis=0)
            weigh,weigh_Debug = self.__get_all2all()
            corr = self.get_corr(weigh)

            return [train_x, train_y, test_x, test_y, [corr,np.array(weigh)]]


        k_len = self.data.shape[0]//k
        start = 0
        while True:

            end = start + k_len
            if self.data.shape[0] - end < k_len:
                end = self.data.shape[0]

            test_index  = [i for i in range(start,end)]
            xy_cross_val_list.append(get_xy_split(test_index))
            start = end

            if end == self.data.shape[0]:
                break

        self.cur_data = self.data

        with open(save_path, 'wb') as f:
            pickle.dump(xy_cross_val_list,f)
        return xy_cross_val_list

    def get_xy_onehot(self,task_path):
        self.__get_data(task_path)
        x = self.data[:, :-1]
        y = tf.keras.utils.to_categorical(self.data[:, -1] - 1)

        return x,y

    def get_xy(self,task_path):
        self.__get_data(task_path)
        x = self.data[:, :-1]
        y = self.data[:, -1]

        return x,y

    def onhot2id(self,y_onehot):
        y_ori_list = np.argmax(y_onehot,axis=1)
        y_id = [y_ori+1 for y_ori in y_ori_list]
        return np.array(y_id)

    def get_xy_onehot_classes(self,task_path,):

        self.__get_data(task_path)

        xy_classes = []

        columns = self.x_name.copy()
        columns.append('cla')
        data = pd.DataFrame(self.data.copy(),columns=columns)

        for i in set(data.iloc[:, -1]):
            x = data.loc[data['cla']==i].iloc[:,:-1]
            y = tf.keras.utils.to_categorical(data.loc[data['cla']==i].iloc[:,-1] - 1,num_classes=len(set(data.iloc[:, -1])))

            xy_classes.append((i,x,y))

        return xy_classes

utils = Utils()

if __name__ == '__main__':
    pass




import importlib
import tensorflow as tf
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from sann.bin import config
from sann.bin import data_utils



class my_layer(tf.keras.layers.Layer):
    def __init__(self,out_num,trainable,initial_weigh):
        super().__init__()
        self.initial_weigh = initial_weigh
        self.trainable = trainable
        self.out_num = out_num
        self.drop_layer = tf.keras.layers.Dropout(config.config.drop_rate)
        self.combination_layer = tf.keras.layers.Dense(out_num)

    def build(self, input_shape):

        if type(self.initial_weigh) != type(None):
            self.kernel_corr = tf.Variable(name='kernel', initial_value=self.initial_weigh[0],
                                      trainable=self.trainable, dtype='float32')
        else:
            self.kernel_corr = self.add_weight(name='kernel',
                                          shape=[input_shape[1], input_shape[1]], trainable=True, dtype='float32')

        if type(self.initial_weigh) != type(None):
            self.kernel_weights = tf.Variable(name='kernel', initial_value=self.initial_weigh[1],
                                      trainable=self.trainable, dtype='float32')
        else:
            self.kernel_weights = self.add_weight(name='kernel',
                                          shape=[input_shape[1], self.out_num], trainable=True, dtype='float32')






    def call(self,input):

        kernel_weights_corr = tf.matmul(self.kernel_corr, self.kernel_weights)


        input = tf.cast(input,tf.float32)


        input = self.drop_layer(input)
        output = tf.matmul(input,kernel_weights_corr)



        return output


class Model(tf.keras.Model):
    def __init__(self,trainable,initial_weigh):
        super().__init__()
        self.layer = my_layer(data_utils.utils.cla_num, trainable=trainable, initial_weigh=initial_weigh)

    def call(self, x):
        x = self.layer(x)
        x = tf.nn.softmax(x,axis=1)
        return x


























import pandas as pd
import os
import warnings
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# 因为在进行编码的时候，会对一些汉字描述的特征进行编码，如：轻度编码为1，重度编码为2。
# 在进行DataFrame的筛选时，就会出现比较 汉字str与编码int的情况，因此会出现警告信息。
# 这个警告信息可以忽略。
warnings.filterwarnings(action='ignore',message='elementwise comparison failed')

class CodingData():
    """
    这是一个编码数据的类，包括将one数据转换为onehot数据，将onehot数据转化为one数据，编码分类信息，编码等级信息等。
    本类中所有函数的参数都以列表形式传入。
    """
    def __init__(self, old_dir):
        self.data = self.data = pd.read_excel(old_dir,header=0)   # 第一行默认为表头，数据第一行必须有表头
        self.data.dropna(axis=0,how='all',inplace=True)  # 删除excel结尾处有可能的出现的无效行
        self.save_dir = os.path.dirname(old_dir)       # 默认保存路径为原数据同一目录

    def onehot2one(self,columns):
        """
        本函数功能是将onehot编码的多列转化为one编码的一列。
        :param columns: 是一个列表，列表元素是需要转化的列名 如 ['column1','column2']
        直接将修改self.data
        """

        y_onehot = self.data.loc[:,columns].copy()   # 将要转化的列取出来
        new_column_name = "+".join(columns)

        # 增添一个列，列名=要转化的列的列名拼接起来，值为0
        y_onehot.loc[:, new_column_name] = pd.Series([0 for i in range(y_onehot.shape[0])])

        # 遍历要转化的每一个列，按照其在列表中的顺序，对其为1的值进行编码。第一列为1的转化为1，第二列为1的转化为2，以此类推
        # 因为onehot编码的数据，一行只有一个1。因此在进行上一行所述的编码操作之后，令新的列的值等onehot列的和
        # 这个和的值就可以代表该行的onehot分类
        for i,y_column in enumerate(columns,start=1):
            y_onehot.loc[y_onehot[y_column]==1,[y_column]] = i   #依次对每一列中为1的值进行编码，数字每次加1
            y_onehot.loc[:, new_column_name] += y_onehot[y_column].values  # one列的值等于之前所有列的和。


        for i, y_column in enumerate(columns,start=1):

            # 将one列的值，从数字编码转化为原onehot编码的列名
            y_onehot.loc[y_onehot[new_column_name]==i,[new_column_name]] = y_column


        self.data.drop(columns,axis=1,inplace=True)         # 删除原onehot列
        self.data.loc[:,new_column_name] = y_onehot.loc[:,new_column_name].copy() # 将新列赋值给self.data

    def one2onehot(self,one_columns):
        """
        这个函数将一列，转化为onehot编码的多列
        :param one_columns:
        :return:
        """

        # 从要转化为onehot的多个列中一次选出一个进行转化
        for one_column in one_columns:

            # 将这个列的值作为转化后onehot列的列名
            columns = set(self.data.loc[:,one_column].values.flatten())

            for column in columns:   # 遍历每一个将要新建的onehot列名
                new_column = one_column + column # 为了便于区分，onehot列名之前要加上原列的列名
                # 用定义的列名，新建一个onehot列，其值为0
                self.data.loc[:,new_column] = pd.Series([0 for i in range(self.data.shape[0])])
                # 筛选出原列中，值为新建列列名的行，在新建列中将该行赋值为1
                self.data.loc[self.data[one_column]==column,[new_column]] = 1

        self.data.drop(one_columns, axis=1, inplace=True)

    def grades22(self,regulations):
        """
        本函数是将分级数据转化为二分类数据，参数是转化规则，本质是一个列表，列表元素是元组如：【（‘轻度’，0），（“重度”，1）】

        """
        # 遍历所有列
        for column in self.data.columns.values:
            for item in regulations:
                # 对每一列，遍历规则，改为2分类数据
                self.data.loc[self.data[column] == item[0],[column]] = item[1]

    def coding_y(self,y_column):
        """
        本函数功能是将y进行编码，并将编码关系储存到数据目录下。参数为一个列表：【‘列名’】
        :param y_column:
        :return:
        """
        y_names = set(self.data.loc[:, y_column].values.flatten())

        coded_relationship = []
        for i,y_name in enumerate(y_names):
            self.data.loc[self.data[y_column[0]]==y_name,[y_column[0]]] = i
            coded_relationship.append((i, y_name))

        save_dir = os.path.join(self.save_dir,'coded_relationship.txt')
        with open(save_dir,'w',encoding='utf-8') as f:
            for item in coded_relationship:
                f.write(str(item[0]) + ': ' + item[1] + '\n')

    def save(self):
        save_path = os.path.join(self.save_dir,'new.xlsx')
        self.data.to_excel(save_path)






import setuptools

with open("README.txt", "r",encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='jhsp',
    version="0.0.32",
    author='Junhe Zhao',
    author_email="zhaozong111@gmail.com",
    description="A package only used by Junhe Zhao",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://pypi.org',
    packages = setuptools.find_packages(),
    install_requires=['pandas==1.1.3',
                        'numpy==1.18.5',
                        'tensorflow==2.3.1',
                        'matplotlib==3.3.2',
                        'scikit_learn==0.23.2',
                      ],

)


import sys
import os

sys.path.append(os.path.dirname(__file__))


# 导入sann模块的所有包
import configparser


# 定义SANN类
# # 在调用时实例化，因为要传入一个工作目录
class Sann():
    def __init__(self,input_path,task_name_list):

        cg = configparser.ConfigParser()
        cg['path'] = {'input_path' : input_path,

                             }

        task_name_list_str = ''
        for task_name in task_name_list:
            task_name_list_str += ','
            task_name_list_str += task_name

        cg['task'] = {'task_name_list': task_name_list_str,
                      'cur_task_path': 'None'
                      }

        with open(os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)),'sann'),'initial.ini'), 'w',encoding='utf-8') as f:
            cg.write(f)

        from jhsp.sann.bin import config
        from jhsp.sann.bin import other_utils
        from jhsp.sann.bin import data_utils
        from jhsp.sann.bin import model_utils

        self.config = config.config
        self.other_utils = other_utils.utils
        self.data_utils = data_utils.utils
        self.model_utils = model_utils.utils

class Preprocessing():

    from jhsp.preprocessing import codingdata
    from jhsp.preprocessing import describingdata
    from jhsp.preprocessing import selectingfeatures
    CodingData = codingdata.CodingData
    DescribingData = describingdata.DescribingData
    SelectingFeatures = selectingfeatures.SelectingFeatures

    def __init__(self):
        pass



class Tools():
    from jhsp.tools import collatingwords
    CollatingWords = collatingwords.CollatingWords

    def __init__(self):
        pass














